@@ -2,6 +2,7 @@
 from __future__ import print_function, unicode_literals
 
 import ctypes
+import os
 import re
 import time
 
@@ -21,25 +22,7 @@ class Fstab(object):
     def __init__(self, log ):
         self.log_func = log
 
-        self.no_sparse = set(
-            [
-                "fuse",  # termux-sdcard
-                "vfat",  # linux-efi
-                "fat32",
-                "fat16",
-                "fat12",
-                "fat-32",
-                "fat-16",
-                "fat-12",
-                "fat 32",
-                "fat 16",
-                "fat 12",
-                "exfat",
-                "ex-fat",
-                "ex fat",
-                "hpfs",  # macos
-            ]
-        )
+        self.trusted = False
         self.tab  = None
         self.cache   = {}
         self.age = 0.0
@@ -47,8 +30,8 @@ class Fstab(object):
     def log(self, msg , c   = 0)  :
         self.log_func("fstab", msg + "\033[K", c)
 
-    def get(self, path ):
-        if time.time() - self.age > 600 or len(self.cache) > 9000:
+    def get(self, path )  :
+        if len(self.cache) > 9000:
             self.age = time.time()
             self.tab = None
             self.cache = {}
@@ -68,6 +51,7 @@ class Fstab(object):
                 self.log(msg.format(path, fs, min_ex()), 3)
                 return fs
 
+        path = path.lstrip("/")
         try:
             return self.cache[path]
         except:
@@ -83,7 +67,7 @@ class Fstab(object):
         self.log("found {} at {}".format(fs, path))
         return fs
 
-    def build_tab(self):
+    def build_tab(self)  :
         self.log("building tab")
 
         sptn = r"^.*? on (.*) type ([^ ]+) \(.*"
@@ -98,7 +82,8 @@ class Fstab(object):
             if not m:
                 continue
 
-            tab1.append(m.groups())
+            zs1, zs2 = m.groups()
+            tab1.append((str(zs1), str(zs2)))
 
         tab1.sort(key=lambda x: (len(x[0]), x[0]))
         path1, fs1 = tab1[0]
@@ -108,18 +93,53 @@ class Fstab(object):
 
         self.tab = tab
 
-    def get_unix(self, path ):
+    def relabel(self, path , nval )  :
+        assert self.tab
+        self.cache = {}
+        path = path.lstrip("/")
+        ptn = re.compile(r"^[^\\/]*")
+        vn, rem = self.tab._find(path)
+        if not self.trusted:
+            # no mtab access; have to build as we go
+            if "/" in rem:
+                self.tab.add("idk", os.path.join(vn.vpath, rem.split("/")[0]))
+            if rem:
+                self.tab.add(nval, path)
+            else:
+                vn.realpath = nval
+
+            return
+
+        visit = [vn]
+        while visit:
+            vn = visit.pop()
+            vn.realpath = ptn.sub(nval, vn.realpath)
+            visit.extend(list(vn.nodes.values()))
+
+    def get_unix(self, path )  :
         if not self.tab:
-            self.build_tab()
+            try:
+                self.build_tab()
+                self.trusted = True
+            except:
+                # prisonparty or other restrictive environment
+                self.log("failed to build tab:\n{}".format(min_ex()), 3)
+                self.tab = VFS(self.log_func, "idk", "/", AXS(), {})
+                self.trusted = False
 
-        return self.tab._find(path)[0].realpath.split("/")[0]
+        assert self.tab
+        ret = self.tab._find(path)[0]
+        if self.trusted or path == ret.vpath:
+            return ret.realpath.split("/")[0]
+        else:
+            return "idk"
 
-    def get_w32(self, path ):
+    def get_w32(self, path )  :
         # list mountpoints: fsutil fsinfo drives
 
         from ctypes.wintypes import BOOL, DWORD, LPCWSTR, LPDWORD, LPWSTR, MAX_PATH
 
-        def echk(rc, fun, args):
+        def echk(rc , fun , args )  :
             if not rc:
                 raise ctypes.WinError(ctypes.get_last_error())
             return None
