@@ -184,6 +184,8 @@ class HttpCli(object):
     def j2s(self, name , **ka )  :
         tpl = self.conn.hsrv.j2[name]
         ka["ts"] = self.conn.hsrv.cachebuster()
+        ka["lang"] = self.args.lang
+        ka["favico"] = self.args.favico
         ka["svcname"] = self.args.doctitle
         ka["html_head"] = self.html_head
         return tpl.render(**ka)  # type: ignore
@@ -378,13 +380,18 @@ class HttpCli(object):
                 if not self._check_nonfatal(pex, post):
                     self.keepalive = False
 
-                msg = str(ex) if pex == ex else min_ex()
+                em = str(ex)
+                msg = em if pex == ex else min_ex()
                 self.log("{}\033[0m, {}".format(msg, self.vpath), 3)
 
-                msg = "{}\r\nURL: {}\r\n".format(str(ex), self.vpath)
+                msg = "{}\r\nURL: {}\r\n".format(em, self.vpath)
                 if self.hint:
                     msg += "hint: {}\r\n".format(self.hint)
 
+                if "database is locked" in em:
+                    self.conn.hsrv.broker.say("log_stacks")
+                    msg += "hint: important info in the server log\r\n"
+
                 msg = "<pre>" + html_escape(msg)
                 self.reply(msg.encode("utf-8", "replace"), status=pex.code, volsan=True)
                 return self.keepalive
@@ -545,6 +552,10 @@ class HttpCli(object):
             static_path = os.path.join(E.mod, "web/", self.vpath[5:])
             return self.tx_file(static_path)
 
+        if "cf_challenge" in self.uparam:
+            self.reply(self.j2s("cf").encode("utf-8", "replace"))
+            return True
+
         if not self.can_read and not self.can_write and not self.can_get:
             if self.vpath:
                 self.log("inaccessible: [{}]".format(self.vpath))
@@ -911,6 +922,9 @@ class HttpCli(object):
         except:
             raise Pebkac(422, "you POSTed invalid json")
 
+        # self.reply(b" DD" + b"oS Protection ", 503)
+        # return True
+
         if "srch" in self.uparam or "srch" in body:
             return self.handle_search(body)
 
@@ -1048,7 +1062,7 @@ class HttpCli(object):
             reader = read_socket(self.sr, remains)
 
             f = None
-            fpool = not self.args.no_fpool and (not ANYWIN or sprs)
+            fpool = not self.args.no_fpool and sprs
             if fpool:
                 with self.mutex:
                     try:
@@ -1118,8 +1132,10 @@ class HttpCli(object):
             except:
                 self.log("failed to utime ({}, {})".format(fin_path, times))
 
+        cinf = self.headers.get("x-up2k-stat", "")
+
         spd = self._spd(post_sz)
-        self.log("{} thank".format(spd))
+        self.log("{:70} thank {}".format(spd, cinf))
         self.reply(b"thank")
         return True
 
@@ -1855,7 +1871,9 @@ class HttpCli(object):
             "edit": "edit" in self.uparam,
             "title": html_escape(self.vpath, crlf=True),
             "lastmod": int(ts_md * 1000),
-            "md_plug": "true" if self.args.emp else "false",
+            "lang": self.args.lang,
+            "favico": self.args.favico,
+            "have_emp": self.args.emp,
             "md_chk_rate": self.args.mcr,
             "md": boundary,
             "arg_base": arg_base,
@@ -2218,7 +2236,8 @@ class HttpCli(object):
             ret = json.dumps(ls)
             mime = "application/json"
 
-        self.reply(ret.encode("utf-8", "replace") + b"\n", mime=mime)
+        ret += "\n\033[0m" if arg == "v" else "\n"
+        self.reply(ret.encode("utf-8", "replace"), mime=mime)
         return True
 
     def tx_browser(self)  :
@@ -2384,6 +2403,7 @@ class HttpCli(object):
             "perms": json.dumps(perms),
             "taglist": [],
             "def_hcols": [],
+            "have_emp": self.args.emp,
             "have_up2k_idx": ("e2d" in vn.flags),
             "have_tags_idx": ("e2t" in vn.flags),
             "have_acode": (not self.args.no_acode),
@@ -2397,11 +2417,21 @@ class HttpCli(object):
             "readme": readme,
             "title": html_escape(self.vpath, crlf=True),
             "srv_info": srv_infot,
-            "lang": self.args.lang,
             "dtheme": self.args.theme,
             "themes": self.args.themes,
             "turbolvl": self.args.turbo,
+            "u2sort": self.args.u2sort,
         }
+
+        if self.args.js_browser:
+            j2a["js"] = self.args.js_browser
+
+        if self.args.css_browser:
+            j2a["css"] = self.args.css_browser
+
+        if not self.conn.hsrv.prism:
+            j2a["no_prism"] = True
+
         if not self.can_read:
             if is_ls:
                 return self.tx_ls(ls_ret)
@@ -2601,9 +2631,6 @@ class HttpCli(object):
             if doctxt is not None:
                 j2a["doc"] = doctxt
 
-        if not self.conn.hsrv.prism:
-            j2a["no_prism"] = True
-
         for d in dirs:
             d["name"] += "/"
 
@@ -2615,19 +2642,12 @@ class HttpCli(object):
         else:
             j2a["files"] = dirs + files
 
-        j2a["logues"] = logues
         j2a["taglist"] = taglist
         j2a["txt_ext"] = self.args.textfiles.replace(",", " ")
 
         if "mth" in vn.flags:
             j2a["def_hcols"] = vn.flags["mth"].split(",")
 
-        if self.args.js_browser:
-            j2a["js"] = self.args.js_browser
-
-        if self.args.css_browser:
-            j2a["css"] = self.args.css_browser
-
         html = self.j2s(tpl, **j2a)
         self.reply(html.encode("utf-8", "replace"))
         return True
