@@ -10,7 +10,7 @@ import sys
 
 from .__init__ import PY2, WINDOWS, unicode
 from .bos import bos
-from .util import REKOBO_LKEY, fsenc, retchk, runcmd, uncyg
+from .util import REKOBO_LKEY, fsenc, min_ex, retchk, runcmd, uncyg
 
 try:
     raise Exception()
@@ -44,7 +44,9 @@ class MParser(object):
 
         self.timeout = 30
         self.force = False
+        self.kill = "t"  # tree; all children recursively
         self.audio = "y"
+        self.pri = 0  # priority; higher = later
         self.ext = []
 
         while True:
@@ -66,6 +68,10 @@ class MParser(object):
                 self.audio = arg[1:]  # [r]equire [n]ot [d]ontcare
                 continue
 
+            if arg.startswith("k"):
+                self.kill = arg[1:]  # [t]ree [m]ain [n]one
+                continue
+
             if arg == "f":
                 self.force = True
                 continue
@@ -78,6 +84,10 @@ class MParser(object):
                 self.ext.append(arg[1:])
                 continue
 
+            if arg.startswith("p"):
+                self.pri = int(arg[1:] or "1")
+                continue
+
             raise Exception()
 
 
@@ -213,7 +223,10 @@ def parse_ffprobe(txt )      :
         fps = ret[".fps"]
         if "/" in fps:
             fa, fb = fps.split("/")
-            fps = int(fa) * 1.0 / int(fb)
+            try:
+                fps = int(fa) * 1.0 / int(fb)
+            except:
+                fps = 9001
 
         if fps < 1000 and fmt.get("format_name") not in ["image2", "png_pipe"]:
             ret[".fps"] = round(fps, 3)
@@ -479,7 +492,9 @@ class MTag(object):
         ret, md = ffprobe(abspath)
         return self.normalize_tags(ret, md)
 
-    def get_bin(self, parsers  , abspath )   :
+    def get_bin(
+        self, parsers  , abspath , oth_tags  
+    )   :
         if not bos.path.isfile(abspath):
             return {}
 
@@ -489,14 +504,19 @@ class MTag(object):
         env = os.environ.copy()
         env["PYTHONPATH"] = pypath
 
-        ret = {}
-        for tagname, parser in parsers.items():
+        ret   = {}
+        for tagname, parser in sorted(parsers.items(), key=lambda x: (x[1].pri, x[0])):
             try:
                 cmd = [parser.bin, abspath]
                 if parser.bin.endswith(".py"):
                     cmd = [sys.executable] + cmd
 
-                args = {"env": env, "timeout": parser.timeout}
+                args = {"env": env, "timeout": parser.timeout, "kill": parser.kill}
+
+                if parser.pri:
+                    zd = oth_tags.copy()
+                    zd.update(ret)
+                    args["sin"] = json.dumps(zd).encode("utf-8", "replace")
 
                 if WINDOWS:
                     args["creationflags"] = 0x4000
@@ -518,6 +538,8 @@ class MTag(object):
                         if tag and tag in zj:
                             ret[tag] = zj[tag]
             except:
-                pass
+                if self.args.mtag_v:
+                    t = "mtag error: tagname {}, parser {}, file {} => {}"
+                    self.log(t.format(tagname, parser.bin, abspath, min_ex()))
 
         return ret
