@@ -2,12 +2,15 @@
 from __future__ import print_function, unicode_literals
 
 import re
-import sys
 import socket
+import sys
 
-from .__init__ import MACOS, ANYWIN, unicode
+from .__init__ import ANYWIN, MACOS, TYPE_CHECKING, unicode
 from .util import chkcmd
 
+if TYPE_CHECKING:
+    from .svchub import SvcHub
+
 
 class TcpSrv(object):
     """
@@ -15,16 +18,16 @@ class TcpSrv(object):
     which then uses the least busy HttpSrv to handle it
     """
 
-    def __init__(self, hub):
+    def __init__(self, hub ):
         self.hub = hub
         self.args = hub.args
         self.log = hub.log
 
         self.stopping = False
 
-        self.srv = []
+        self.srv  = []
         self.nsrv = 0
-        ok = {}
+        ok   = {}
         for ip in self.args.i:
             ok[ip] = []
             for port in self.args.p:
@@ -34,8 +37,8 @@ class TcpSrv(object):
                     ok[ip].append(port)
                 except Exception as ex:
                     if self.args.ign_ebind or self.args.ign_ebind_all:
-                        m = "could not listen on {}:{}: {}"
-                        self.log("tcpsrv", m.format(ip, port, ex), c=3)
+                        t = "could not listen on {}:{}: {}"
+                        self.log("tcpsrv", t.format(ip, port, ex), c=3)
                     else:
                         raise
 
@@ -55,15 +58,21 @@ class TcpSrv(object):
                     eps[x] = "external"
 
         msgs = []
-        title_tab = {}
+        title_tab    = {}
         title_vars = [x[1:] for x in self.args.wintitle.split(" ") if x.startswith("$")]
-        m = "available @ http://{}:{}/  (\033[33m{}\033[0m)"
+        t = "available @ {}://{}:{}/  (\033[33m{}\033[0m)"
         for ip, desc in sorted(eps.items(), key=lambda x: x[1]):
             for port in sorted(self.args.p):
                 if port not in ok.get(ip, ok.get("0.0.0.0", [])):
                     continue
 
-                msgs.append(m.format(ip, port, desc))
+                proto = " http"
+                if self.args.http_only:
+                    pass
+                elif self.args.https_only or port == 443:
+                    proto = "https"
+
+                msgs.append(t.format(proto, ip, port, desc))
 
                 if not self.args.wintitle:
                     continue
@@ -92,13 +101,13 @@ class TcpSrv(object):
 
         if msgs:
             msgs[-1] += "\n"
-            for m in msgs:
-                self.log("tcpsrv", m)
+            for t in msgs:
+                self.log("tcpsrv", t)
 
         if self.args.wintitle:
             self._set_wintitle(title_tab)
 
-    def _listen(self, ip, port):
+    def _listen(self, ip , port )  :
         srv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         srv.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         srv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
@@ -114,7 +123,7 @@ class TcpSrv(object):
                 raise
             raise Exception(e)
 
-    def run(self):
+    def run(self)  :
         for srv in self.srv:
             srv.listen(self.args.nc)
             ip, port = srv.getsockname()
@@ -124,9 +133,9 @@ class TcpSrv(object):
             if self.args.q:
                 print(msg)
 
-            self.hub.broker.put(False, "listen", srv)
+            self.hub.broker.say("listen", srv)
 
-    def shutdown(self):
+    def shutdown(self)  :
         self.stopping = True
         try:
             for srv in self.srv:
@@ -136,25 +145,59 @@ class TcpSrv(object):
 
         self.log("tcpsrv", "ok bye")
 
-    def ips_linux(self):
-        eps = {}
+    def ips_linux_ifconfig(self)   :
+        # for termux
+        try:
+            txt, _ = chkcmd(["ifconfig"])
+        except:
+            return {}
+
+        eps   = {}
+        dev = None
+        ip = None
+        up = None
+        for ln in (txt + "\n").split("\n"):
+            if not ln.strip() and dev and ip:
+                eps[ip] = dev + ("" if up else ", \033[31mLINK-DOWN")
+                dev = ip = up = None
+                continue
+
+            if ln == ln.lstrip():
+                dev = re.split(r"[: ]", ln)[0]
+
+            if "UP" in re.split(r"[<>, \t]", ln):
+                up = True
+
+            m = re.match(r"^\s+inet\s+([^ ]+)", ln)
+            if m:
+                ip = m.group(1)
+
+        return eps
+
+    def ips_linux(self)   :
         try:
             txt, _ = chkcmd(["ip", "addr"])
         except:
-            return eps
+            return self.ips_linux_ifconfig()
 
         r = re.compile(r"^\s+inet ([^ ]+)/.* (.*)")
+        ri = re.compile(r"^\s*[0-9]+\s*:.*")
+        up = False
+        eps   = {}
         for ln in txt.split("\n"):
+            if ri.match(ln):
+                up = "UP" in re.split("[>,< ]", ln)
+
             try:
-                ip, dev = r.match(ln.rstrip()).groups()
-                eps[ip] = dev
+                ip, dev = r.match(ln.rstrip()).groups()  # type: ignore
+                eps[ip] = dev + ("" if up else ", \033[31mLINK-DOWN")
             except:
                 pass
 
         return eps
 
-    def ips_macos(self):
-        eps = {}
+    def ips_macos(self)   :
+        eps   = {}
         try:
             txt, _ = chkcmd(["ifconfig"])
         except:
@@ -162,7 +205,7 @@ class TcpSrv(object):
 
         rdev = re.compile(r"^([^ ]+):")
         rip = re.compile(r"^\tinet ([0-9\.]+) ")
-        dev = None
+        dev = "UNKNOWN"
         for ln in txt.split("\n"):
             m = rdev.match(ln)
             if m:
@@ -171,34 +214,46 @@ class TcpSrv(object):
             m = rip.match(ln)
             if m:
                 eps[m.group(1)] = dev
-                dev = None
+                dev = "UNKNOWN"
 
         return eps
 
-    def ips_windows_ipconfig(self):
-        eps = {}
+    def ips_windows_ipconfig(self)    :
+        eps   = {}
+        offs  = set()
         try:
             txt, _ = chkcmd(["ipconfig"])
         except:
-            return eps
+            return eps, offs
 
         rdev = re.compile(r"(^[^ ].*):$")
         rip = re.compile(r"^ +IPv?4? [^:]+: *([0-9\.]{7,15})$")
+        roff = re.compile(r".*: Media disconnected$")
         dev = None
         for ln in txt.replace("\r", "").split("\n"):
             m = rdev.match(ln)
             if m:
+                if dev and dev not in eps.values():
+                    offs.add(dev)
+
                 dev = m.group(1).split(" adapter ", 1)[-1]
 
+            if dev and roff.match(ln):
+                offs.add(dev)
+                dev = None
+
             m = rip.match(ln)
             if m and dev:
                 eps[m.group(1)] = dev
                 dev = None
 
-        return eps
+        if dev and dev not in eps.values():
+            offs.add(dev)
 
-    def ips_windows_netsh(self):
-        eps = {}
+        return eps, offs
+
+    def ips_windows_netsh(self)   :
+        eps   = {}
         try:
             txt, _ = chkcmd("netsh interface ip show address".split())
         except:
@@ -215,16 +270,18 @@ class TcpSrv(object):
             m = rip.match(ln)
             if m and dev:
                 eps[m.group(1)] = dev
-                dev = None
 
         return eps
 
-    def detect_interfaces(self, listen_ips):
+    def detect_interfaces(self, listen_ips )   :
         if MACOS:
             eps = self.ips_macos()
         elif ANYWIN:
-            eps = self.ips_windows_ipconfig()  # sees more interfaces
+            eps, off = self.ips_windows_ipconfig()  # sees more interfaces + link state
             eps.update(self.ips_windows_netsh())  # has better names
+            for k, v in eps.items():
+                if v in off:
+                    eps[k] += ", \033[31mLINK-DOWN"
         else:
             eps = self.ips_linux()
 
@@ -243,7 +300,6 @@ class TcpSrv(object):
         ]:
             try:
                 s.connect((ip, 1))
-                # raise OSError(13, "a")
                 default_route = s.getsockname()[0]
                 break
             except (OSError, socket.error) as ex:
@@ -264,23 +320,23 @@ class TcpSrv(object):
 
         return eps
 
-    def _set_wintitle(self, vars):
-        vars["all"] = vars.get("all", {"Local-Only": 1})
-        vars["pub"] = vars.get("pub", vars["all"])
+    def _set_wintitle(self, vs   )  :
+        vs["all"] = vs.get("all", {"Local-Only": 1})
+        vs["pub"] = vs.get("pub", vs["all"])
 
-        vars2 = {}
-        for k, eps in vars.items():
-            vars2[k] = {
+        vs2 = {}
+        for k, eps in vs.items():
+            vs2[k] = {
                 ep: 1
                 for ep in eps.keys()
                 if ":" not in ep or ep.split(":")[0] not in eps
             }
 
         title = ""
-        vars = vars2
+        vs = vs2
         for p in self.args.wintitle.split(" "):
             if p.startswith("$"):
-                p = " and ".join(sorted(vars.get(p[1:], {"(None)": 1}).keys()))
+                p = " and ".join(sorted(vs.get(p[1:], {"(None)": 1}).keys()))
 
             title += "{} ".format(p)
 
