@@ -9,6 +9,7 @@ import math
 import os
 import re
 import shutil
+import signal
 import stat
 import subprocess as sp
 import threading
@@ -35,6 +36,7 @@ from .util import (
     quotep,
     ren_open,
     rmdirs,
+    rmdirs_up,
     s2hms,
     s3dec,
     s3enc,
@@ -67,12 +69,20 @@ class Dbw(object):
 
 
 class Mpqe(object):
-    def __init__(self, mtp  , entags , w , abspath ):
+    def __init__(
+        self,
+        mtp  ,
+        entags ,
+        w ,
+        abspath ,
+        oth_tags  ,
+    ):
         # mtp empty = mtag
         self.mtp = mtp
         self.entags = entags
         self.w = w
         self.abspath = abspath
+        self.oth_tags = oth_tags
 
 
 class Up2k(object):
@@ -86,7 +96,9 @@ class Up2k(object):
         self.r_hash = re.compile("^[0-9a-zA-Z_-]{44}$")
 
         self.gid = 0
+        self.stop = False
         self.mutex = threading.Lock()
+        self.blocked  = None
         self.pp  = None
         self.rescan_cond = threading.Condition()
         self.need_rescan  = set()
@@ -106,6 +118,7 @@ class Up2k(object):
         self.pending_tags      = []
         self.hashq      = Queue()
         self.tagq     = Queue()
+        self.tag_event = threading.Condition()
         self.n_hashq = 0
         self.n_tagq = 0
         self.mpool_used = False
@@ -181,6 +194,14 @@ class Up2k(object):
     def log(self, msg , c   = 0)  :
         self.log_func("up2k", msg + "\033[K", c)
 
+    def _block(self, why )  :
+        self.blocked = why
+        self.log("uploads are temporarily blocked due to " + why, 3)
+
+    def _unblock(self)  :
+        self.blocked = None
+        self.log("uploads are now possible", 2)
+
     def get_state(self)  :
         mtpq   = 0
         q = "select count(w) from mt where k = 't:mtp'"
@@ -404,7 +425,13 @@ class Up2k(object):
                 self.mtag = None
 
         # e2ds(a) volumes first
+        if next((zv for zv in vols if "e2ds" in zv.flags), None):
+            self._block("indexing")
+
         for vol in vols:
+            if self.stop:
+                break
+
             en  = set()
             if "mte" in vol.flags:
                 en = set(vol.flags["mte"].split(","))
@@ -420,16 +447,51 @@ class Up2k(object):
                 if vac:
                     need_vac[vol] = True
 
-            if "e2ts" not in vol.flags:
-                t = "online, idle"
-            else:
+            if "e2v" in vol.flags:
+                t = "online (integrity-check pending)"
+            elif "e2ts" in vol.flags:
                 t = "online (tags pending)"
+            else:
+                t = "online, idle"
 
             self.volstate[vol.vpath] = t
 
+        # file contents verification
+        if next((zv for zv in vols if "e2v" in zv.flags), None):
+            self._block("integrity verification")
+
+        for vol in vols:
+            if self.stop:
+                break
+
+            if "e2v" not in vol.flags:
+                continue
+
+            t = "online (verifying integrity)"
+            self.volstate[vol.vpath] = t
+            self.log("{} [{}]".format(t, vol.realpath))
+
+            nmod = self._verify_integrity(vol)
+            if nmod:
+                self.log("modified {} entries in the db".format(nmod), 3)
+                need_vac[vol] = True
+
+            if "e2ts" in vol.flags:
+                t = "online (tags pending)"
+            else:
+                t = "online, idle"
+
+            self.volstate[vol.vpath] = t
+
+        if self.blocked:
+            self._unblock()
+
         # open the rest + do any e2ts(a)
         needed_mutagen = False
         for vol in vols:
+            if self.stop:
+                break
+
             if "e2ts" not in vol.flags:
                 continue
 
@@ -454,6 +516,9 @@ class Up2k(object):
                 cur.connection.commit()
                 cur.execute("vacuum")
 
+        if self.stop:
+            return False
+
         self.pp.end = True
 
         msg = "{} volumes in {:.2f} sec"
@@ -516,7 +581,9 @@ class Up2k(object):
             if vpath:
                 vpath += "/"
 
-            self.log("/{} {}".format(vpath, " ".join(sorted(a))), "35")
+            zs = " ".join(sorted(a))
+            zs = zs.replace("30mre.compile(", "30m(")  # nohash
+            self.log("/{} {}".format(vpath, zs), "35")
 
         reg = {}
         drp = None
@@ -633,6 +700,9 @@ class Up2k(object):
         gl = sorted(g)
         inames = {x[0]: 1 for x in gl}
         for iname, inf in gl:
+            if self.stop:
+                return -1
+
             abspath = os.path.join(cdir, iname)
             if rei and rei.search(abspath):
                 continue
@@ -711,7 +781,9 @@ class Up2k(object):
                         self.log("file: {}".format(abspath))
 
                     try:
-                        hashes = self._hashlist_from_file(abspath)
+                        hashes = self._hashlist_from_file(
+                            abspath, "a{}, ".format(self.pp.n)
+                        )
                     except Exception as ex:
                         self.log("hash: {} @ [{}]".format(repr(ex), abspath))
                         continue
@@ -728,6 +800,9 @@ class Up2k(object):
                     db.n = 0
                     db.t = time.time()
 
+        if self.stop:
+            return -1
+
         # drop missing files
         rd = cdir[len(top) + 1 :].strip("/")
         if WINDOWS:
@@ -788,6 +863,106 @@ class Up2k(object):
 
         return n_rm
 
+    def _verify_integrity(self, vol )  :
+        """expensive; blocks database access until finished"""
+        ptop = vol.realpath
+        assert self.pp and self.mtag
+
+        cur = self.cur[ptop]
+        rei = vol.flags.get("noidx")
+        reh = vol.flags.get("nohash")
+        e2vu = "e2vu" in vol.flags
+        e2vp = "e2vp" in vol.flags
+
+        excl = [
+            d[len(vol.vpath) :].lstrip("/")
+            for d in self.asrv.vfs.all_vols
+            if d != vol.vpath and (d.startswith(vol.vpath + "/") or not vol.vpath)
+        ]
+        qexa  = []
+        pexa  = []
+        for vpath in excl:
+            qexa.append("up.rd != ? and not up.rd like ?||'%'")
+            pexa.extend([vpath, vpath])
+
+        pex = tuple(pexa)
+        qex = " and ".join(qexa)
+        if qex:
+            qex = " where " + qex
+
+        rewark      = []
+
+        with self.mutex:
+            b_left = 0
+            n_left = 0
+            q = "select sz from up" + qex
+            for (sz,) in cur.execute(q, pex):
+                b_left += sz  # sum() can overflow according to docs
+                n_left += 1
+
+            q = "select w, mt, sz, rd, fn from up" + qex
+            for w, mt, sz, drd, dfn in cur.execute(q, pex):
+                if self.stop:
+                    return -1
+
+                n_left -= 1
+                b_left -= sz
+                if drd.startswith("//") or dfn.startswith("//"):
+                    rd, fn = s3dec(drd, dfn)
+                else:
+                    rd = drd
+                    fn = dfn
+
+                abspath = os.path.join(ptop, rd, fn)
+                if rei and rei.search(abspath):
+                    continue
+
+                nohash = reh.search(abspath) if reh else False
+
+                pf = "v{}, {:.0f}+".format(n_left, b_left / 1024 / 1024)
+                self.pp.msg = pf + abspath
+
+                st = bos.stat(abspath)
+                sz2 = st.st_size
+                mt2 = int(st.st_mtime)
+
+                if nohash:
+                    w2 = up2k_wark_from_metadata(self.salt, sz2, mt2, rd, fn)
+                else:
+                    if sz2 > 1024 * 1024 * 32:
+                        self.log("file: {}".format(abspath))
+
+                    try:
+                        hashes = self._hashlist_from_file(abspath, pf)
+                    except Exception as ex:
+                        self.log("hash: {} @ [{}]".format(repr(ex), abspath))
+                        continue
+
+                    w2 = up2k_wark_from_hashlist(self.salt, sz2, hashes)
+
+                if w == w2:
+                    continue
+
+                rewark.append((drd, dfn, w2, sz2, mt2))
+
+                t = "hash mismatch: {}\n  db: {} ({} byte, {})\n  fs: {} ({} byte, {})"
+                t = t.format(abspath, w, sz, mt, w2, sz2, mt2)
+                self.log(t, 1)
+
+            if e2vp and rewark:
+                self.hub.retcode = 1
+                os.kill(os.getpid(), signal.SIGTERM)
+                raise Exception("{} files have incorrect hashes".format(len(rewark)))
+
+            if not e2vu:
+                return 0
+
+            for rd, fn, w, sz, mt in rewark:
+                q = "update up set w = ?, sz = ?, mt = ? where rd = ? and fn = ? limit 1"
+                cur.execute(q, (w, sz, int(mt), rd, fn))
+
+        return len(rewark)
+
     def _build_tags_index(self, vol )    :
         ptop = vol.realpath
         with self.mutex:
@@ -854,6 +1029,9 @@ class Up2k(object):
             c3 = conn.cursor()
             n_left = cur.execute("select count(w) from up").fetchone()[0]
             for w, rd, fn in cur.execute("select w, rd, fn from up order by rd, fn"):
+                if self.stop:
+                    return -1, -1, False
+
                 n_left -= 1
                 q = "select w from mt where w = ?"
                 if c2.execute(q, (w[:16],)).fetchone():
@@ -871,7 +1049,7 @@ class Up2k(object):
                 if not mpool:
                     n_tags = self._tag_file(c3, entags, w, abspath)
                 else:
-                    mpool.put(Mpqe({}, entags, w, abspath))
+                    mpool.put(Mpqe({}, entags, w, abspath, {}))
                     # not registry cursor; do not self.mutex:
                     n_tags = len(self._flush_mpool(c3))
 
@@ -958,6 +1136,7 @@ class Up2k(object):
         to_delete = {}
         in_progress = {}
         while True:
+            did_nothing = True
             with self.mutex:
                 if gid != self.gid:
                     break
@@ -967,31 +1146,40 @@ class Up2k(object):
                 warks = [str(x[0]) for x in zq]
                 jobs = []
                 for w in warks:
+                    if w in in_progress:
+                        continue
+
                     q = "select rd, fn from up where substr(w,1,16)=? limit 1"
                     rd, fn = cur.execute(q, (w,)).fetchone()
                     rd, fn = s3dec(rd, fn)
                     abspath = os.path.join(ptop, rd, fn)
 
                     q = "select k from mt where w = ?"
-                    zq2 = cur.execute(q, (w,)).fetchall()
-                    have    = {x[0]: 1 for x in zq2}
+                    zq = cur.execute(q, (w,)).fetchall()
+                    have    = {x[0]: 1 for x in zq}
 
+                    did_nothing = False
                     parsers = self._get_parsers(ptop, have, abspath)
                     if not parsers:
                         to_delete[w] = True
                         n_left -= 1
                         continue
 
-                    if w in in_progress:
-                        continue
+                    if next((x for x in parsers.values() if x.pri), None):
+                        q = "select k, v from mt where w = ?"
+                        zq2 = cur.execute(q, (w,)).fetchall()
+                        oth_tags = {str(k): v for k, v in zq2}
+                    else:
+                        oth_tags = {}
 
-                    jobs.append(Mpqe(parsers, set(), w, abspath))
+                    jobs.append(Mpqe(parsers, set(), w, abspath, oth_tags))
                     in_progress[w] = True
 
             with self.mutex:
                 done = self._flush_mpool(wcur)
                 for w in done:
                     to_delete[w] = True
+                    did_nothing = False
                     in_progress.pop(w)
                     n_done += 1
 
@@ -1004,6 +1192,10 @@ class Up2k(object):
             if not warks:
                 break
 
+            if did_nothing:
+                with self.tag_event:
+                    self.tag_event.wait(0.2)
+
             if not jobs:
                 continue
 
@@ -1104,7 +1296,7 @@ class Up2k(object):
             return
 
         for _ in range(mpool.maxsize):
-            mpool.put(Mpqe({}, set(), "", ""))
+            mpool.put(Mpqe({}, set(), "", "", {}))
 
         mpool.join()
 
@@ -1120,7 +1312,7 @@ class Up2k(object):
                 if not qe.mtp:
                     tags = self.mtag.get(qe.abspath)
                 else:
-                    tags = self.mtag.get_bin(qe.mtp, qe.abspath)
+                    tags = self.mtag.get_bin(qe.mtp, qe.abspath, qe.oth_tags)
                     vtags = [
                         "\033[36m{} \033[33m{}".format(k, v) for k, v in tags.items()
                     ]
@@ -1132,6 +1324,10 @@ class Up2k(object):
             except:
                 ex = traceback.format_exc()
                 self._log_tag_err(qe.mtp or self.mtag.backend, qe.abspath, ex)
+            finally:
+                if qe.mtp:
+                    with self.tag_event:
+                        self.tag_event.notify_all()
 
             q.task_done()
 
@@ -1307,11 +1503,24 @@ class Up2k(object):
 
         cur.connection.commit()
 
+    def _job_volchk(self, cj  )  :
+        if not self.register_vpath(cj["ptop"], cj["vcfg"]):
+            if cj["ptop"] not in self.registry:
+                raise Pebkac(410, "location unavailable")
+
     def handle_json(self, cj  )   :
-        with self.mutex:
-            if not self.register_vpath(cj["ptop"], cj["vcfg"]):
-                if cj["ptop"] not in self.registry:
-                    raise Pebkac(410, "location unavailable")
+        try:
+            # bit expensive; 3.9=10x 3.11=2x
+            if self.mutex.acquire(timeout=10):
+                self._job_volchk(cj)
+                self.mutex.release()
+            else:
+                t = "cannot receive uploads right now;\nserver busy with {}.\nPlease wait; the client will retry..."
+                raise Pebkac(503, t.format(self.blocked or "[unknown]"))
+        except TypeError:
+            # py2
+            with self.mutex:
+                self._job_volchk(cj)
 
         cj["name"] = sanitize_fn(cj["name"], "", [".prologue.html", ".epilogue.html"])
         cj["poke"] = time.time()
@@ -1326,7 +1535,7 @@ class Up2k(object):
 
         # check if filesystem supports sparse files;
         # refuse out-of-order / multithreaded uploading if sprs False
-        sprs = self.fstab.get(pdir) not in self.fstab.no_sparse
+        sprs = self.fstab.get(pdir) != "ng"
 
         with self.mutex:
             cur = self.cur.get(cj["ptop"])
@@ -1359,7 +1568,7 @@ class Up2k(object):
                         "prel": dp_dir,
                         "vtop": cj["vtop"],
                         "ptop": cj["ptop"],
-                        "sprs": sprs,
+                        "sprs": sprs,  # dontcare; finished anyways
                         "size": dsize,
                         "lmod": dtime,
                         "addr": ip,
@@ -1494,7 +1703,7 @@ class Up2k(object):
                 "purl": purl,
                 "size": job["size"],
                 "lmod": job["lmod"],
-                "sprs": sprs,
+                "sprs": job.get("sprs", sprs),
                 "hash": job["need"],
                 "wark": wark,
             }
@@ -1830,11 +2039,12 @@ class Up2k(object):
         adir, fn = os.path.split(atop)
         try:
             st = bos.lstat(atop)
+            is_dir = stat.S_ISDIR(st.st_mode)
         except:
             raise Pebkac(400, "file not found on disk (already deleted?)")
 
         scandir = not self.args.no_scandir
-        if stat.S_ISDIR(st.st_mode):
+        if is_dir:
             g = vn.walk("", rem, [], uname, permsets, True, scandir, True)
             if unpost:
                 raise Pebkac(400, "cannot unpost folders")
@@ -1869,8 +2079,14 @@ class Up2k(object):
 
                 bos.unlink(abspath)
 
-        rm = rmdirs(self.log_func, scandir, True, atop, 1)
-        return n_files, rm[0], rm[1]
+        ok  = []
+        ng  = []
+        if is_dir:
+            ok, ng = rmdirs(self.log_func, scandir, True, atop, 1)
+
+        ok2, ng2 = rmdirs_up(os.path.dirname(atop))
+
+        return n_files, ok + ok2, ng + ng2
 
     def handle_mv(self, uname , svp , dvp )  :
         svn, srem = self.asrv.vfs.get(svp, uname, True, False, True)
@@ -1912,6 +2128,7 @@ class Up2k(object):
                     self._mv_file(uname, svpf, dvpf)
 
         rmdirs(self.log_func, scandir, True, sabs, 1)
+        rmdirs_up(os.path.dirname(sabs))
         return "k"
 
     def _mv_file(self, uname , svp , dvp )  :
@@ -2168,14 +2385,15 @@ class Up2k(object):
 
         return wark
 
-    def _hashlist_from_file(self, path )  :
+    def _hashlist_from_file(self, path , prefix  = "")  :
         fsz = bos.path.getsize(path)
         csz = up2k_chunksize(fsz)
         ret = []
         with open(fsenc(path), "rb", 512 * 1024) as f:
             while fsz > 0:
                 if self.pp:
-                    self.pp.msg = "{} MB, {}".format(int(fsz / 1024 / 1024), path)
+                    mb = int(fsz / 1024 / 1024)
+                    self.pp.msg = "{}{} MB, {}".format(prefix, mb, path)
 
                 hashobj = hashlib.sha512()
                 rem = min(csz, fsz)
@@ -2195,8 +2413,11 @@ class Up2k(object):
         return ret
 
     def _new_upload(self, job  )  :
-        self.registry[job["ptop"]][job["wark"]] = job
         pdir = os.path.join(job["ptop"], job["prel"])
+        if not job["size"] and bos.path.isfile(os.path.join(pdir, job["name"])):
+            return
+
+        self.registry[job["ptop"]][job["wark"]] = job
         job["name"] = self._untaken(pdir, job["name"], job["t0"], job["addr"])
         # if len(job["name"].split(".")) > 8:
         #    raise Exception("aaa")
@@ -2215,20 +2436,45 @@ class Up2k(object):
         suffix = "-{:.6f}-{}".format(job["t0"], dip)
         with ren_open(tnam, "wb", fdir=pdir, suffix=suffix) as zfw:
             f, job["tnam"] = zfw["orz"]
+            abspath = os.path.join(pdir, job["tnam"])
+            sprs = job["sprs"]
+            sz = job["size"]
+            relabel = False
             if (
                 ANYWIN
-                and job["sprs"]
+                and sprs
                 and self.args.sparse
-                and self.args.sparse * 1024 * 1024 <= job["size"]
+                and self.args.sparse * 1024 * 1024 <= sz
             ):
-                fp = os.path.join(pdir, job["tnam"])
                 try:
-                    sp.check_call(["fsutil", "sparse", "setflag", fp])
+                    sp.check_call(["fsutil", "sparse", "setflag", abspath])
                 except:
-                    self.log("could not sparse [{}]".format(fp), 3)
+                    self.log("could not sparse [{}]".format(abspath), 3)
+                    relabel = True
+                    sprs = False
 
-            if job["hash"] and job["sprs"]:
-                f.seek(job["size"] - 1)
+            if not ANYWIN and sprs and sz > 1024 * 1024:
+                fs = self.fstab.get(pdir)
+                if fs != "ok":
+                    relabel = True
+                    f.seek(1024 * 1024 - 1)
+                    f.write(b"e")
+                    f.flush()
+                    try:
+                        nblk = bos.stat(abspath).st_blocks
+                        sprs = nblk < 2048
+                    except:
+                        sprs = False
+
+            if relabel:
+                t = "sparse files {} on {} filesystem at {}"
+                nv = "ok" if sprs else "ng"
+                self.log(t.format(nv, self.fstab.get(pdir), pdir))
+                self.fstab.relabel(pdir, nv)
+                job["sprs"] = sprs
+
+            if job["hash"] and sprs:
+                f.seek(sz - 1)
                 f.write(b"e")
 
         if not job["hash"]:
@@ -2300,6 +2546,9 @@ class Up2k(object):
                 except:
                     pass
 
+        if self.args.nw:
+            return
+
         path = os.path.join(histpath, "up2k.snap")
         if not reg:
             if ptop not in self.snap_prev or self.snap_prev[ptop] is not None:
@@ -2346,7 +2595,7 @@ class Up2k(object):
                 ntags1 = len(tags)
                 parsers = self._get_parsers(ptop, tags, abspath)
                 if parsers:
-                    tags.update(self.mtag.get_bin(parsers, abspath))
+                    tags.update(self.mtag.get_bin(parsers, abspath, tags))
             except Exception as ex:
                 self._log_tag_err("", abspath, ex)
                 continue
@@ -2400,6 +2649,7 @@ class Up2k(object):
         # self.log("hashq {} push {}/{}/{}".format(self.n_hashq, ptop, rd, fn))
 
     def shutdown(self)  :
+        self.stop = True
         self.log("writing snapshot")
         self.do_snapshot()
 
