@@ -30,6 +30,12 @@ try:
 except:
     HAVE_SQLITE3 = False
 
+try:
+    HAVE_PSUTIL = True
+    import psutil
+except:
+    HAVE_PSUTIL = False
+
 try:
     raise Exception()
     raise Exception()
@@ -121,6 +127,7 @@ HTTPCODE = {
     429: "Too Many Requests",
     500: "Internal Server Error",
     501: "Not Implemented",
+    503: "Service Unavailable",
 }
 
 
@@ -130,6 +137,9 @@ IMPLICATIONS = [
     ["e2tsr", "e2ts"],
     ["e2ts", "e2t"],
     ["e2t", "e2d"],
+    ["e2vu", "e2v"],
+    ["e2vp", "e2v"],
+    ["e2v", "e2d"],
 ]
 
 
@@ -589,6 +599,7 @@ def ren_open(
         ext = bname[ofs:] + ext
         bname = bname[:ofs]
 
+    asciified = False
     b64 = ""
     while True:
         try:
@@ -614,11 +625,20 @@ def ren_open(
 
         except OSError as ex_:
             ex = ex_
+
+            if ex.errno == 22 and not asciified:
+                asciified = True
+                bname, fname = [
+                    zs.encode("ascii", "replace").decode("ascii").replace("?", "_")
+                    for zs in [bname, fname]
+                ]
+                continue
+
             if ex.errno not in [36, 63] and (not WINDOWS or ex.errno != 22):
                 raise
 
         if not b64:
-            zs = (bname + ext).encode("utf-8", "replace")
+            zs = (orig_name + "\n" + suffix).encode("utf-8", "replace")
             zs = hashlib.sha512(zs).digest()[:12]
             b64 = base64.urlsafe_b64encode(zs).decode("utf-8")
 
@@ -1405,7 +1425,8 @@ def statdir(
 def rmdirs(
     logger , scandir , lstat , top , depth 
 )   :
-    if not os.path.exists(fsenc(top)) or not os.path.isdir(fsenc(top)):
+    """rmdir all descendants, then self"""
+    if not os.path.isdir(fsenc(top)):
         top = os.path.dirname(top)
         depth -= 1
 
@@ -1429,6 +1450,21 @@ def rmdirs(
     return ok, ng
 
 
+def rmdirs_up(top )   :
+    """rmdir on self, then all parents"""
+    try:
+        os.rmdir(fsenc(top))
+    except:
+        return [], [top]
+
+    par = os.path.dirname(top)
+    if not par:
+        return [top], []
+
+    ok, ng = rmdirs_up(par)
+    return [top] + ok, ng
+
+
 def unescape_cookie(orig )  :
     # mw=idk; doot=qwe%2Crty%3Basd+fgh%2Bjkl%25zxc%26vbn  # qwe,rty;asd fgh+jkl%zxc&vbn
     ret = ""
@@ -1479,22 +1515,109 @@ def guess_mime(url , fallback  = "application/octet-stream")  :
     return ret
 
 
+def getalive(pids , pgid )  :
+    alive = []
+    for pid in pids:
+        try:
+            if pgid:
+                # check if still one of ours
+                if os.getpgid(pid) == pgid:
+                    alive.append(pid)
+            else:
+                # windows doesn't have pgroups; assume
+                psutil.Process(pid)
+                alive.append(pid)
+        except:
+            pass
+
+    return alive
+
+
+def killtree(root )  :
+    """still racy but i tried"""
+    try:
+        # limit the damage where possible (unixes)
+        pgid = os.getpgid(os.getpid())
+    except:
+        pgid = 0
+
+    if HAVE_PSUTIL:
+        pids = [root]
+        parent = psutil.Process(root)
+        for child in parent.children(recursive=True):
+            pids.append(child.pid)
+            child.terminate()
+        parent.terminate()
+        parent = None
+    elif pgid:
+        # linux-only
+        pids = []
+        chk = [root]
+        while chk:
+            pid = chk[0]
+            chk = chk[1:]
+            pids.append(pid)
+            _, t, _ = runcmd(["pgrep", "-P", str(pid)])
+            chk += [int(x) for x in t.strip().split("\n") if x]
+
+        pids = getalive(pids, pgid)  # filter to our pgroup
+        for pid in pids:
+            os.kill(pid, signal.SIGTERM)
+    else:
+        # windows gets minimal effort sorry
+        os.kill(pid, signal.SIGTERM)
+        return
+
+    for n in range(10):
+        time.sleep(0.1)
+        pids = getalive(pids, pgid)
+        if not pids or n > 3 and pids == [root]:
+            break
+
+    for pid in pids:
+        try:
+            os.kill(pid, signal.SIGKILL)
+        except:
+            pass
+
+
 def runcmd(
     argv  , timeout  = None, **ka 
 )    :
+    kill = ka.pop("kill", "t")  # [t]ree [m]ain [n]one
+
+    sin = ka.pop("sin", None)
+    if sin:
+        ka["stdin"] = sp.PIPE
+
     p = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE, **ka)
     if not timeout or PY2:
-        stdout, stderr = p.communicate()
+        stdout, stderr = p.communicate(sin)
     else:
         try:
-            stdout, stderr = p.communicate(timeout=timeout)
+            stdout, stderr = p.communicate(sin, timeout=timeout)
         except sp.TimeoutExpired:
-            p.kill()
-            stdout, stderr = p.communicate()
+            if kill == "n":
+                return -18, "", ""  # SIGCONT; leave it be
+            elif kill == "m":
+                p.kill()
+            else:
+                killtree(p.pid)
+
+            try:
+                stdout, stderr = p.communicate(timeout=1)
+            except:
+                stdout = b""
+                stderr = b""
 
     stdout = stdout.decode("utf-8", "replace")
     stderr = stderr.decode("utf-8", "replace")
-    return p.returncode, stdout, stderr
+
+    rc = p.returncode
+    if rc is None:
+        rc = -14  # SIGALRM; failed to kill
+
+    return rc, stdout, stderr
 
 
 def chkcmd(argv  , **ka )   :
@@ -1695,16 +1818,15 @@ def termsize()   :
     # from hashwalk
     env = os.environ
 
-    def ioctl_GWINSZ(fd):
+    def ioctl_GWINSZ(fd )   :
         try:
             import fcntl
-            import struct
             import termios
 
-            cr = struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, "1234"))
+            cr = struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, b"1234"))
+            return int(cr[1]), int(cr[0])
         except:
-            return
-        return cr
+            return None
 
     cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
     if not cr:
@@ -1715,13 +1837,13 @@ def termsize()   :
         except:
             pass
 
-    if not cr:
-        try:
-            cr = (env["LINES"], env["COLUMNS"])
-        except:
-            cr = (25, 80)
+    if cr:
+        return cr
 
-    return int(cr[1]), int(cr[0])
+    try:
+        return int(env["COLUMNS"]), int(env["LINES"])
+    except:
+        return 80, 25
 
 
 class Pebkac(Exception):
