@@ -15,6 +15,7 @@ function goto_up2k() {
 // chrome requires https to use crypto.subtle,
 // usually it's undefined but some chromes throw on invoke
 var up2k = null,
+    up2k_hooks = [],
     sha_js = window.WebAssembly ? 'hw' : 'ac',  // ff53,c57,sa11
     m = 'will use ' + sha_js + ' instead of native sha512 due to';
 
@@ -135,14 +136,19 @@ function up2k_flagbus() {
 }
 
 
-function U2pvis(act, btns, uc) {
+function U2pvis(act, btns, uc, st) {
     var r = this;
     r.act = act;
     r.ctr = { "ok": 0, "ng": 0, "bz": 0, "q": 0 };
     r.tab = [];
+    r.hq = {};
     r.head = 0;
     r.tail = -1;
     r.wsz = 3;
+    r.npotato = 99;
+    r.modn = 0;
+    r.modv = 0;
+    r.mod0 = null;
 
     var markup = {
         '404': '<span class="err">404</span>',
@@ -180,6 +186,9 @@ function U2pvis(act, btns, uc) {
     };
 
     r.is_act = function (card) {
+        if (uc.potato && !uc.fsearch)
+            return false;
+
         if (r.act == "done")
             return card == "ok" || card == "ng";
 
@@ -196,11 +205,14 @@ function U2pvis(act, btns, uc) {
         if (!r.is_act(fo.in))
             return;
 
-        var obj = ebi('f{0}{1}'.format(nfile, field.slice(1)));
+        var k = 'f{0}{1}'.format(nfile, field.slice(1)),
+            obj = ebi(k);
+
         obj.innerHTML = field == 'ht' ? (markup[html] || html) : html;
         if (field == 'hp') {
             obj.style.color = '';
             obj.style.background = '';
+            delete r.hq[nfile];
         }
     };
 
@@ -244,12 +256,9 @@ function U2pvis(act, btns, uc) {
         if (!r.is_act(fo.in))
             return;
 
-        var obj = ebi('f{0}p'.format(fobj.n)),
-            o1 = p[0] - 2, o2 = p[0] - 0.1, o3 = p[0];
+        var o1 = p[0] - 2, o2 = p[0] - 0.1, o3 = p[0];
 
-        obj.innerHTML = fo.hp;
-        obj.style.color = '#fff';
-        obj.style.background = 'linear-gradient(90deg, #025, #06a ' + o1 + '%, #09d ' + o2 + '%, #222 ' + o3 + '%, #222 99%, #555)';
+        r.hq[fobj.n] = [fo.hp, '#fff', 'linear-gradient(90deg, #025, #06a ' + o1 + '%, #09d ' + o2 + '%, #222 ' + o3 + '%, #222 99%, #555)'];
     };
 
     r.prog = function (fobj, nchunk, cbd) {
@@ -304,9 +313,7 @@ function U2pvis(act, btns, uc) {
             throw new Error('see console');
         }
 
-        obj.innerHTML = fo.hp;
-        obj.style.color = '#fff';
-        obj.style.background = 'linear-gradient(90deg, #050, #270 ' + o1 + '%, #4b0 ' + o2 + '%, #222 ' + o3 + '%, #222 99%, #555)';
+        r.hq[fobj.n] = [fo.hp, '#fff', 'linear-gradient(90deg, #050, #270 ' + o1 + '%, #4b0 ' + o2 + '%, #222 ' + o3 + '%, #222 99%, #555)'];
     };
 
     r.move = function (nfile, newcat) {
@@ -320,6 +327,10 @@ function U2pvis(act, btns, uc) {
         fo.in = newcat;
         r.ctr[oldcat]--;
         r.ctr[newcat]++;
+
+        while (st.car < r.tab.length && has(['ok', 'ng'], r.tab[st.car].in))
+            st.car++;
+
         r.drawcard(oldcat);
         r.drawcard(newcat);
         if (r.is_act(newcat)) {
@@ -342,7 +353,10 @@ function U2pvis(act, btns, uc) {
     };
 
     r.bzw = function () {
-        var first = QS('#u2tab>tbody>tr:first-child');
+        var mod = 0,
+            t0 = Date.now(),
+            first = QS('#u2tab>tbody>tr:first-child');
+
         if (!first)
             return;
 
@@ -352,14 +366,131 @@ function U2pvis(act, btns, uc) {
 
         while (r.head - first > r.wsz) {
             qsr('#f' + (first++));
+            mod++;
         }
-        while (last - r.tail < r.wsz && last < r.tab.length - 2) {
+        while (last - r.tail < r.wsz && last < r.tab.length - 1) {
             var obj = ebi('f' + (++last));
-            if (!obj)
+            if (!obj) {
                 r.addrow(last);
+                mod++;
+            }
+        }
+        if (mod && r.modn < 200 && ebi('repl').offsetTop) {
+            if (++r.modn >= 10) {
+                if (r.modn == 10)
+                    r.mod0 = Date.now();
+
+                r.modv += Date.now() - t0;
+            }
+
+            if (r.modn >= 200) {
+                var n = r.modn - 10,
+                    ipu = r.modv / n,
+                    spu = (Date.now() - r.mod0) / n,
+                    ir = spu / ipu;
+
+                console.log('bzw:', f2f(ipu, 2), ' spu:', f2f(spu, 2), ' ir:', f2f(ir, 2), ' tab:', r.tab.length);
+                // efficiency estimates;
+                // ir: 5=16% 4=50%,30% 27=100%
+                // ipu: 2.7=16% 2=30% 1.6=50% 1.8=100% (ng for big files)
+                if (ipu >= 1.5 && ir <= 9 && r.tab.length >= 1000 && r.tab[Math.floor(r.tab.length / 3)].bt <= 1024 * 1024 * 4)
+                    r.go_potato();
+            }
         }
     };
 
+    r.potatolabels = function () {
+        var ode = ebi('u2depotato'),
+            oen = ebi('u2enpotato');
+
+        if (!ode)
+            return;
+
+        ode.style.display = uc.potato ? '' : 'none';
+        oen.style.display = uc.potato ? 'none' : '';
+    }
+
+    r.potato = function () {
+        ebi('u2tabw').style.minHeight = '';
+        QS('#u2cards a[act="bz"]').click();
+        timer[uc.potato ? "add" : "rm"](draw_potato);
+        timer[uc.potato ? "rm" : "add"](apply_html);
+        r.potatolabels();
+    };
+
+    r.go_potato = function () {
+        r.go_potato = noop;
+        var ode = mknod('div', 'u2depotato'),
+            oen = mknod('div', 'u2enpotato'),
+            u2f = ebi('u2foot'),
+            btn = ebi('potato');
+
+        ode.innerHTML = L.u_depot;
+        oen.innerHTML = L.u_enpot;
+
+        if (sread('potato') === null) {
+            btn.click();
+            toast.inf(30, L.u_gotpot);
+            localStorage.removeItem('potato');
+        }
+
+        u2f.appendChild(ode);
+        u2f.appendChild(oen);
+        ode.onclick = oen.onclick = btn.onclick;
+        r.potatolabels();
+    };
+
+    function draw_potato() {
+        if (++r.npotato < 2)
+            return;
+
+        r.npotato = 0;
+        var html = [
+            "<p>files: &nbsp; <b>{0}</b> finished, &nbsp; <b>{1}</b> failed, &nbsp; <b>{2}</b> busy, &nbsp; <b>{3}</b> queued</p>".format(r.ctr.ok, r.ctr.ng, r.ctr.bz, r.ctr.q),
+        ];
+
+        while (r.head < r.tab.length && has(["ok", "ng"], r.tab[r.head].in))
+            r.head++;
+
+        var act = null;
+        if (r.head < r.tab.length)
+            act = r.tab[r.head];
+
+        if (act)
+            html.push("<p>file {0} of {1} : &nbsp; {2} &nbsp; <code>{3}</code></p>\n<div>{4}</div>".format(r.head + 1, r.tab.length, act.ht, act.hp, act.hn));
+
+        html = html.join('\n');
+        if (r.hpotato == html)
+            return;
+
+        r.hpotato = html;
+        ebi('u2mu').innerHTML = html;
+    }
+
+    function apply_html() {
+        var oq = {}, n = 0;
+        for (var k in r.hq) {
+            var o = ebi('f{0}p'.format(k));
+            if (!o)
+                continue;
+
+            oq[k] = o;
+            n++;
+        }
+        if (!n)
+            return;
+
+        for (var k in oq) {
+            var o = oq[k],
+                v = r.hq[k];
+
+            o.innerHTML = v[0];
+            o.style.color = v[1];
+            o.style.background = v[2];
+        }
+        r.hq = {};
+    }
+
     r.drawcard = function (cat) {
         var cards = QSA('#u2cards>a>span');
 
@@ -441,8 +572,7 @@ function U2pvis(act, btns, uc) {
         if (as_html)
             return '<tr id="f' + nfile + '">' + ret + '</tr>';
 
-        var obj = mknod('tr');
-        obj.setAttribute('id', 'f' + nfile);
+        var obj = mknod('tr', 'f' + nfile);
         obj.innerHTML = ret;
         return obj;
     };
@@ -476,6 +606,7 @@ function U2pvis(act, btns, uc) {
     }
 
     r.changecard(r.act);
+    r.potato();
 }
 
 
@@ -527,7 +658,7 @@ function Donut(uc, st) {
     }
 
     r.on = function (ya) {
-        r.fc = r.tc = 99;
+        r.fc = r.tc = r.dc = 99;
         r.eta = null;
         r.base = pos();
         optab.innerHTML = ya ? svg() : optab.getAttribute('ico');
@@ -543,10 +674,16 @@ function Donut(uc, st) {
 
         var t = st.bytes.total - r.base,
             v = pos() - r.base,
-            ofs = el.style.strokeDashoffset = o - o * v / t;
+            ofs = o - o * v / t;
+
+        if (!uc.potato || ++r.dc >= 4) {
+            el.style.strokeDashoffset = ofs;
+            r.dc = 0;
+        }
 
         if (++r.tc >= 10) {
-            wintitle(f2f(v * 100 / t, 1) + '%, ' + r.eta + 's, ', true);
+            wintitle("{0}%, {1}s, #{2}, ".format(
+                f2f(v * 100 / t, 1), r.eta, st.files.length - st.nfile.upload), true);
             r.tc = 0;
         }
 
@@ -578,6 +715,12 @@ function fsearch_explain(n) {
 
 
 function up2k_init(subtle) {
+    var r = {
+        "init_deps": init_deps,
+        "set_fsearch": set_fsearch,
+        "gotallfiles": [gotallfiles]  // hooks
+    };
+
     function showmodal(msg) {
         ebi('u2notbtn').innerHTML = msg;
         ebi('u2btn').style.display = 'none';
@@ -612,7 +755,10 @@ function up2k_init(subtle) {
                 m = L.u_ancient;
                 setmsg('');
             }
-            ebi('u2foot').innerHTML = '<big>' + m + '</big>';
+            qsr('#u2depmsg');
+            var o = mknod('div', 'u2depmsg');
+            o.innerHTML = m;
+            ebi('u2foot').appendChild(o);
         }
         loading_deps = true;
     }
@@ -647,17 +793,24 @@ function up2k_init(subtle) {
     var parallel_uploads = icfg_get('nthread'),
         uc = {},
         fdom_ctr = 0,
-        min_filebuf = 0;
+        min_filebuf = 0,
+        biggest_file = 0;
 
     bcfg_bind(uc, 'multitask', 'multitask', true, null, false);
+    bcfg_bind(uc, 'potato', 'potato', false, set_potato, false);
     bcfg_bind(uc, 'ask_up', 'ask_up', true, null, false);
     bcfg_bind(uc, 'flag_en', 'flag_en', false, apply_flag_cfg);
     bcfg_bind(uc, 'fsearch', 'fsearch', false, set_fsearch, false);
     bcfg_bind(uc, 'turbo', 'u2turbo', turbolvl > 1, draw_turbo, false);
     bcfg_bind(uc, 'datechk', 'u2tdate', turbolvl < 3, null, false);
+    bcfg_bind(uc, 'az', 'u2sort', u2sort.indexOf('n') + 1, set_u2sort, false);
 
     var st = {
         "files": [],
+        "nfile": {
+            "hash": 0,
+            "upload": 0
+        },
         "seen": {},
         "todo": {
             "head": [],
@@ -681,7 +834,11 @@ function up2k_init(subtle) {
             "hashing": 0,
             "uploading": 0,
             "busy": 0
-        }
+        },
+        "car": 0,
+        "modn": 0,
+        "modv": 0,
+        "mod0": null
     };
 
     function push_t(arr, t) {
@@ -693,9 +850,13 @@ function up2k_init(subtle) {
             });
     }
 
-    var pvis = new U2pvis("bz", '#u2cards', uc),
+    var pvis = new U2pvis("bz", '#u2cards', uc, st),
         donut = new Donut(uc, st);
 
+    r.ui = pvis;
+    r.st = st;
+    r.uc = uc;
+
     var bobslice = null;
     if (window.File)
         bobslice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice;
@@ -808,6 +969,8 @@ function up2k_init(subtle) {
         if (err)
             return modal.alert('sorry, ' + err);
 
+        toast.inf(0, 'Scanning files...');
+
         if ((dz == 'up_dz' && uc.fsearch) || (dz == 'srch_dz' && !uc.fsearch))
             tgl_fsearch();
 
@@ -906,7 +1069,8 @@ function up2k_init(subtle) {
 
         if (!dirs.length) {
             if (!pf.length)
-                return gotallfiles(good, nil, bad);
+                // call first hook, pass list of remaining hooks to call
+                return r.gotallfiles[0](good, nil, bad, r.gotallfiles.slice(1));
 
             console.log("retry pf, " + pf.length);
             setTimeout(function () {
@@ -954,6 +1118,12 @@ function up2k_init(subtle) {
     }
 
     function gotallfiles(good_files, nil_files, bad_files) {
+        if (toast.txt == 'Scanning files...')
+            toast.hide();
+
+        if (uc.fsearch && !uc.turbo)
+            nil_files = [];
+
         var ntot = good_files.concat(nil_files, bad_files).length;
         if (bad_files.length) {
             var msg = L.u_badf.format(bad_files.length, ntot);
@@ -1002,6 +1172,13 @@ function up2k_init(subtle) {
         var evpath = get_evpath(),
             draw_each = good_files.length < 50;
 
+        if (!uc.az)
+            good_files.sort(function (a, b) {
+                a = a[0].size;
+                b = b[0].size;
+                return a < b ? -1 : a > b ? 1 : 0;
+            });
+
         for (var a = 0; a < good_files.length; a++) {
             var fobj = good_files[a][0],
                 name = good_files[a][1],
@@ -1032,6 +1209,9 @@ function up2k_init(subtle) {
             if (uc.fsearch)
                 entry.srch = 1;
 
+            if (biggest_file < entry.size)
+                biggest_file = entry.size;
+
             try {
                 if (st.seen[fdir][key])
                     continue;
@@ -1047,9 +1227,9 @@ function up2k_init(subtle) {
                     entry.purl + uricom_enc(entry.name)).join(' '),
                 '📐 ' + L.u_hashing,
                 ''
-            ], fobj.size, draw_each);
+            ], entry.size, draw_each);
 
-            st.bytes.total += fobj.size;
+            st.bytes.total += entry.size;
             st.files.push(entry);
             if (!entry.size)
                 push_t(st.todo.handshake, entry);
@@ -1062,6 +1242,7 @@ function up2k_init(subtle) {
             pvis.drawcard("q");
             pvis.changecard(pvis.act);
         }
+        ebi('u2tabw').className = 'ye';
     }
 
     function more_one_file() {
@@ -1073,7 +1254,7 @@ function up2k_init(subtle) {
     }
     more_one_file();
 
-    var etaref = 0, etaskip = 0, utw_minh = 0;
+    var etaref = 0, etaskip = 0, utw_minh = 0, utw_read = 0;
     function etafun() {
         var nhash = st.busy.head.length + st.busy.hash.length + st.todo.head.length + st.todo.hash.length,
             nsend = st.busy.upload.length + st.todo.upload.length,
@@ -1086,6 +1267,11 @@ function up2k_init(subtle) {
 
         //ebi('acc_info').innerHTML = humantime(st.time.busy) + ' ' + f2f(now / 1000, 1);
 
+        if (++utw_read >= 20) {
+            utw_read = 0;
+            utw_minh = parseInt(ebi('u2tabw').style.minHeight || '0');
+        }
+
         var minh = QS('#op_up2k.act') && st.is_busy ? Math.max(utw_minh, ebi('u2tab').offsetHeight + 32) : 0;
         if (utw_minh < minh || !utw_minh) {
             utw_minh = minh;
@@ -1093,11 +1279,11 @@ function up2k_init(subtle) {
         }
 
         if (!nhash)
-            ebi('u2etah').innerHTML = L.u_etadone.format(humansize(st.bytes.hashed), pvis.ctr["ok"] + pvis.ctr["ng"]);
+            ebi('u2etah').innerHTML = L.u_etadone.format(humansize(st.bytes.hashed), pvis.ctr.ok + pvis.ctr.ng);
 
         if (!nsend && !nhash)
             ebi('u2etau').innerHTML = ebi('u2etat').innerHTML = (
-                L.u_etadone.format(humansize(st.bytes.uploaded), pvis.ctr["ok"] + pvis.ctr["ng"]));
+                L.u_etadone.format(humansize(st.bytes.uploaded), pvis.ctr.ok + pvis.ctr.ng));
 
         if (!st.busy.hash.length && !hashing_permitted())
             nhash = 0;
@@ -1183,9 +1369,14 @@ function up2k_init(subtle) {
             return false;
 
         if (uc.multitask) {
-            var ahead = st.bytes.hashed - st.bytes.finished;
-            return ahead < 1024 * 1024 * 1024 * 4 &&
-                st.todo.handshake.length + st.busy.handshake.length < 16;
+            if (!uc.az)
+                return st.todo.handshake.length + st.busy.handshake.length < 2;
+
+            var ahead = st.bytes.hashed - st.bytes.finished,
+                nmax = ahead < biggest_file / 8 ? 32 : 16;
+
+            return ahead < biggest_file &&
+                st.todo.handshake.length + st.busy.handshake.length < nmax;
         }
         return handshakes_permitted() && 0 ==
             st.todo.handshake.length +
@@ -1212,16 +1403,7 @@ function up2k_init(subtle) {
             running = true;
             while (true) {
                 var now = Date.now(),
-                    oldest_active = Math.min(  // gzip take the wheel
-                        st.todo.head.length ? st.todo.head[0].n : st.files.length,
-                        st.todo.hash.length ? st.todo.hash[0].n : st.files.length,
-                        st.todo.upload.length ? st.todo.upload[0].nfile : st.files.length,
-                        st.todo.handshake.length ? st.todo.handshake[0].n : st.files.length,
-                        st.busy.head.length ? st.busy.head[0].n : st.files.length,
-                        st.busy.hash.length ? st.busy.hash[0].n : st.files.length,
-                        st.busy.upload.length ? st.busy.upload[0].nfile : st.files.length,
-                        st.busy.handshake.length ? st.busy.handshake[0].n : st.files.length),
-                    is_busy = oldest_active < st.files.length;
+                    is_busy = st.car < st.files.length;
 
                 if (was_busy && !is_busy) {
                     for (var a = 0; a < st.files.length; a++) {
@@ -1249,25 +1431,7 @@ function up2k_init(subtle) {
                     donut.on(is_busy);
 
                     if (!is_busy) {
-                        var sr = uc.fsearch,
-                            ok = pvis.ctr["ok"],
-                            ng = pvis.ctr["ng"],
-                            t = uc.ask_up ? 0 : 10;
-
-                        if (ok && ng)
-                            toast.warn(t, (sr ? L.ur_sm : L.ur_um).format(ok, ng));
-                        else if (ok > 1)
-                            toast.ok(t, (sr ? L.ur_aso : L.ur_auo).format(ok));
-                        else if (ok)
-                            toast.ok(t, sr ? L.ur_1so : L.ur_1uo);
-                        else if (ng > 1)
-                            toast.err(t, (sr ? L.ur_asn : L.ur_aun).format(ng));
-                        else if (ng)
-                            toast.err(t, sr ? L.ur_1sn : L.ur_1un);
-
-                        timer.rm(etafun);
-                        timer.rm(donut.do);
-                        utw_minh = 0;
+                        uptoast();
                     }
                     else {
                         timer.add(donut.do);
@@ -1320,7 +1484,7 @@ function up2k_init(subtle) {
 
                 if (st.todo.head.length &&
                     st.busy.head.length < parallel_uploads &&
-                    (!is_busy || st.todo.head[0].n - oldest_active < parallel_uploads * 2)) {
+                    (!is_busy || st.todo.head[0].n - st.car < parallel_uploads * 2)) {
                     exec_head();
                     mou_ikkai = true;
                 }
@@ -1345,6 +1509,31 @@ function up2k_init(subtle) {
                     mou_ikkai = true;
                 }
 
+                if (is_busy && st.modn < 100) {
+                    var t0 = Date.now() + (ebi('repl').offsetTop ? 0 : 0);
+
+                    if (++st.modn >= 10) {
+                        if (st.modn == 10)
+                            st.mod0 = Date.now();
+
+                        st.modv += Date.now() - t0;
+                    }
+
+                    if (st.modn >= 100) {
+                        var n = st.modn - 10,
+                            ipu = st.modv / n,
+                            spu = (Date.now() - st.mod0) / n,
+                            ir = spu / ipu;
+
+                        console.log('tsk:', f2f(ipu, 2), ' spu:', f2f(spu, 2), ' ir:', f2f(ir, 2));
+                        // efficiency estimates;
+                        // ir: 8=16% 11=60% 16=90% 24=100%
+                        // ipu: 1=40% .8=60% .3=100%
+                        if (ipu >= 0.5 && ir <= 15)
+                            pvis.go_potato();
+                    }
+                }
+
                 if (!mou_ikkai || crashed)
                     return defer();
             }
@@ -1353,6 +1542,30 @@ function up2k_init(subtle) {
         return taskerd;
     })();
 
+    function uptoast() {
+        var sr = uc.fsearch,
+            ok = pvis.ctr.ok,
+            ng = pvis.ctr.ng,
+            t = uc.ask_up ? 0 : 10;
+
+        console.log('toast', ok, ng);
+
+        if (ok && ng)
+            toast.warn(t, (sr ? L.ur_sm : L.ur_um).format(ok, ng));
+        else if (ok > 1)
+            toast.ok(t, (sr ? L.ur_aso : L.ur_auo).format(ok));
+        else if (ok)
+            toast.ok(t, sr ? L.ur_1so : L.ur_1uo);
+        else if (ng > 1)
+            toast.err(t, (sr ? L.ur_asn : L.ur_aun).format(ng));
+        else if (ng)
+            toast.err(t, sr ? L.ur_1sn : L.ur_1un);
+
+        timer.rm(etafun);
+        timer.rm(donut.do);
+        utw_minh = 0;
+    }
+
     function chill(t) {
         var now = Date.now();
         if ((t.coolmul || 0) < 2 || now - t.cooldown < t.coolmul * 700)
@@ -1428,6 +1641,7 @@ function up2k_init(subtle) {
     function exec_hash() {
         var t = st.todo.hash.shift();
         st.busy.hash.push(t);
+        st.nfile.hash = t.n;
 
         var bpend = 0,
             nchunk = 0,
@@ -1465,7 +1679,7 @@ function up2k_init(subtle) {
                 hash_calc(nch, e.target.result);
             }
             reader.onload = function (e) {
-                try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+                try { orz(e); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
             };
             reader.onerror = function () {
                 var err = reader.error + '';
@@ -1498,7 +1712,7 @@ function up2k_init(subtle) {
         var hash_calc = function (nch, buf) {
             while (segm_next());
 
-            var hash_done = function (hashbuf) {
+            var orz = function (hashbuf) {
                 var hslice = new Uint8Array(hashbuf).subarray(0, 33),
                     b64str = buf2b64(hslice);
 
@@ -1524,6 +1738,10 @@ function up2k_init(subtle) {
                 tasker();
             };
 
+            var hash_done = function (hashbuf) {
+                try { orz(hashbuf); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
+            };
+
             if (subtle)
                 subtle.digest('SHA-512', buf).then(hash_done);
             else setTimeout(function () {
@@ -1591,7 +1809,7 @@ function up2k_init(subtle) {
             tasker();
         };
         xhr.onload = function (e) {
-            try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+            try { orz(e); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
         };
 
         xhr.open('HEAD', t.purl + uricom_enc(t.name) + '?raw', true);
@@ -1747,6 +1965,8 @@ function up2k_init(subtle) {
                         f2f(spd1, 2), isNaN(spd2) ? '--' : f2f(spd2, 2)));
 
                     pvis.move(t.n, 'ok');
+                    if (!pvis.ctr.bz && !pvis.ctr.q)
+                        uptoast();
                 }
                 else {
                     if (t.t_uploaded)
@@ -1805,7 +2025,7 @@ function up2k_init(subtle) {
             }
         }
         xhr.onload = function (e) {
-            try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+            try { orz(e); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
         };
 
         var req = {
@@ -1844,6 +2064,7 @@ function up2k_init(subtle) {
     function exec_upload() {
         var upt = st.todo.upload.shift();
         st.busy.upload.push(upt);
+        st.nfile.upload = upt.nfile;
 
         var npart = upt.npart,
             t = st.files[upt.nfile],
@@ -1891,12 +2112,15 @@ function up2k_init(subtle) {
             tasker();
         }
         function do_send() {
-            var xhr = new XMLHttpRequest();
+            var xhr = new XMLHttpRequest(),
+                bfin = Math.floor(st.bytes.finished / 1024 / 1024),
+                btot = Math.floor(st.bytes.total / 1024 / 1024);
+
             xhr.upload.onprogress = function (xev) {
                 pvis.prog(t, npart, xev.loaded);
             };
             xhr.onload = function (xev) {
-                try { orz(xhr); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+                try { orz(xhr); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
             };
             xhr.onerror = function (xev) {
                 if (crashed)
@@ -1911,6 +2135,8 @@ function up2k_init(subtle) {
             xhr.open('POST', t.purl, true);
             xhr.setRequestHeader("X-Up2k-Hash", t.hash[npart]);
             xhr.setRequestHeader("X-Up2k-Wark", t.wark);
+            xhr.setRequestHeader("X-Up2k-Stat", "{0}/{1}/{2}/{3} {4}/{5}".format(
+                pvis.ctr.ok, pvis.ctr.ng, pvis.ctr.bz, pvis.ctr.q, btot, btot - bfin));
             xhr.setRequestHeader('Content-Type', 'application/octet-stream');
             if (xhr.overrideMimeType)
                 xhr.overrideMimeType('Content-Type', 'application/octet-stream');
@@ -1942,7 +2168,7 @@ function up2k_init(subtle) {
             ebi('u2conf').className = ebi('u2cards').className = ebi('u2etaw').className = wide;
         }
 
-        wide = write && wem > 78 ? 'ww' : wide;
+        wide = write && wem > 82 ? 'ww' : wide;
         parent = ebi(wide == 'ww' && write ? 'u2c3w' : 'u2c3t');
         var its = [ebi('u2etaw'), ebi('u2cards')];
         if (its[0].parentNode !== parent) {
@@ -2016,24 +2242,27 @@ function up2k_init(subtle) {
     }
 
     function draw_turbo() {
-        var msg = uc.fsearch ? L.u_ts : L.u_tu,
-            omsg = uc.fsearch ? L.u_tu : L.u_ts,
-            html = ebi('u2foot').innerHTML,
-            ohtml = html;
+        var msg = (turbolvl || !uc.turbo) ? null : uc.fsearch ? L.u_ts : L.u_tu,
+            html = ebi('u2foot').innerHTML;
 
-        if (turbolvl || !uc.turbo)
-            msg = null;
+        if (msg && html.indexOf(msg) + 1)
+            return;
 
-        if (msg && html.indexOf(msg) === -1)
-            html = html.replace(omsg, '') + msg;
-        else if (!msg)
-            html = html.replace(L.u_tu, '').replace(L.u_ts, '');
+        qsr('#u2turbomsg');
+        if (!msg)
+            return;
 
-        if (html !== ohtml)
-            ebi('u2foot').innerHTML = html;
+        var o = mknod('div', 'u2turbomsg');
+        o.innerHTML = msg;
+        ebi('u2foot').appendChild(o);
     }
     draw_turbo();
 
+    function set_potato() {
+        pvis.potato();
+        set_fsearch();
+    }
+
     function set_fsearch(new_state) {
         var fixed = false;
 
@@ -2072,6 +2301,10 @@ function up2k_init(subtle) {
 
         ebi('u2tab').className = (uc.fsearch ? 'srch ' : 'up ') + pvis.act;
 
+        var potato = uc.potato && !uc.fsearch;
+        ebi('u2cards').style.display = ebi('u2tab').style.display = potato ? 'none' : '';
+        ebi('u2mu').style.display = potato ? '' : 'none';
+
         draw_turbo();
         onresize();
     }
@@ -2095,9 +2328,12 @@ function up2k_init(subtle) {
         }
     }
 
-    function nop(e) {
-        ev(e);
-        this.click();
+    function set_u2sort() {
+        if (u2sort.indexOf('f') < 0)
+            return;
+
+        bcfg_set('u2sort', uc.az = u2sort.indexOf('n') + 1);
+        localStorage.removeItem('u2sort');
     }
 
     ebi('nthread_add').onclick = function (e) {
@@ -2122,7 +2358,12 @@ function up2k_init(subtle) {
     if (parallel_uploads < 1)
         bumpthread(1);
 
-    return { "init_deps": init_deps, "set_fsearch": set_fsearch, "ui": pvis, "st": st, "uc": uc }
+    setTimeout(function () {
+        for (var a = 0; a < up2k_hooks.length; a++)
+            up2k_hooks[a]();
+    }, 1);
+
+    return r;
 }
 
 
@@ -2138,8 +2379,8 @@ favico.init();
 ebi('ico1').onclick = function () {
     var a = favico.txt == this.textContent;
     swrite('icot', a ? 'c' : this.textContent);
-    swrite('icof', a ? null : '000');
-    swrite('icob', a ? null : '');
+    swrite('icof', a ? 'fc5' : '000');
+    swrite('icob', a ? '222' : '');
     favico.init();
 };
 
