@@ -90,7 +90,10 @@ def get_ramdisk():
 
 
 class NullBroker(object):
-    def put(*args):
+    def say(*args):
+        pass
+
+    def ask(*args):
         pass
 
 
@@ -128,7 +131,7 @@ class VHttpSrv(object):
 class VHttpConn(object):
     def __init__(self, args, asrv, log, buf):
         self.s = VSock(buf)
-        self.sr = Unrecv(self.s)
+        self.sr = Unrecv(self.s, None)
         self.addr = ("127.0.0.1", "42069")
         self.args = args
         self.asrv = asrv
